library("igraph")
library("MASS")
library("Matrix")
library("RSpectra")

#Where to find edge file and store output
setwd("/Users/msoelvsten/dropbox/discreteAKM/VenetoPublicUse/csv")

#The file containing all workers, their employment relationships and weights in the weakly identified linear combinations
Workers <- read.delim("old_All_VenetoTABLE_2_Leave2_2019_6_20_22_56_42MIKKEL.csv", header=FALSE)

names(Workers) <- c("workerID","from","to","old","weight1","weight2")

#Construct the edge file containing only movers and highlight if they are ``high weight'' observations in the weakly identified linear combinations
Edges <- Workers[which(Workers$from != Workers$to),]
#Find the observations with a high weight (above 0.001)
Edges$high1 <- (Edges$weight1^2 >= 0.001)
Edges$high2 <- (Edges$weight2^2 >= 0.001)
#Number of high weight edges
sum(Edges$high1 + Edges$high2>=1)

#Calculate firms sizes and assign cluster status based on whether they disconnect from main graph when high weight observations are removed
Firms <- aggregate( c(Workers$old,Workers$old), by = list( firmID=c(Workers$from,Workers$to) ), sum )
names(Firms) <- c("firmID","size")

#Find the cluster assignments for firms
Firms$cluster1 <- components( graph_from_data_frame(Edges[which(Edges$high1 == 0),2:3], directed = FALSE, vertices = Firms) )$membership
Firms$cluster1 <- Firms$cluster1 !=which.max(
  components( graph_from_data_frame(Edges[which(Edges$high1 == 0),2:3], directed = FALSE, vertices = Firms) )$csize
)
Firms$cluster2 <- components( graph_from_data_frame(Edges[which(Edges$high2 == 0),2:3], directed = FALSE, vertices = Firms) )$membership
Firms$cluster2 <- Firms$cluster2 !=which.max(
  components( graph_from_data_frame(Edges[which(Edges$high2 == 0),2:3], directed = FALSE, vertices = Firms) )$csize
)
Firms$cluster0 <- as.logical((Firms$cluster1==0)*(Firms$cluster2==0))


#Determine if an edge is a move within a cluster or between two clusters
cluster1 <- Firms$firmID[Firms$cluster1]
Edges$cluster1from <- Edges$from %in% cluster1
Edges$cluster1to <- Edges$to %in% cluster1
cluster2 <- Firms$firmID[Firms$cluster2]
Edges$cluster2from <- Edges$from %in% cluster2
Edges$cluster2to <- Edges$to %in% cluster2
Edges$between <- (Edges$cluster1from != Edges$cluster1to) + (Edges$cluster2from != Edges$cluster2to)

#Note that between edges are also high-weight edges, but some higher weight edges can be within cluster
sum(Edges$between)
sum(Edges$between*(Edges$high1 + Edges$high2>=1))

#Construct the graph object
net <- graph_from_data_frame(Edges[,2:3], directed = FALSE, vertices = Firms)

#Graphical parameters
cols <- rainbow(3)
HighlightEdge <- Edges$between
E(net)$arrow.size <- .01
E(net)$width <- .2 + .4*HighlightEdge
E(net)$curved <- 0.1
E(net)$color <- c(cols,"black")[1 + (Edges$cluster1from==1) * (Edges$cluster1to==1) * (HighlightEdge==0) 
                                + 2*(Edges$cluster2from==1) * (Edges$cluster2to==1) * (HighlightEdge==0) + 3*HighlightEdge]

V(net)$label <- NA
V(net)$size = .002*Firms$size
V(net)$color <- cols[1 + Firms$cluster1 + 2*Firms$cluster2]
V(net)$frame.color <- cols[1 + Firms$cluster1 + 2*Firms$cluster2]

#Layout is somewhat random but the weights pull firms in the same clusters together
l <- layout_with_drl(net, weights = 1 - Edges$between )

#Increase distance between clusters and rotate so largest cluster is to the right and two smaller clusters to the left
#Add random noise to locations in the smaller cluster, so it is easier to gauge their size.
l[Firms$cluster0,1] <- (l[Firms$cluster0,1] - mean(l[Firms$cluster0,1]))/sd(l[Firms$cluster0,1]) + 2.5
l[Firms$cluster0,2] <- (l[Firms$cluster0,2] - mean(l[Firms$cluster0,2]))/sd(l[Firms$cluster0,2])
l[Firms$cluster1,1] <- - 1 - runif(sum(Firms$cluster1))
l[Firms$cluster1,2] <- .5 + runif(sum(Firms$cluster1))
l[Firms$cluster2,1] <- - 1 - .5*runif(sum(Firms$cluster2))
l[Firms$cluster2,2] <- -.5 - .5*runif(sum(Firms$cluster2))


#Preliminary plot of network
plot( net , layout = l)


# Saving the network plot that only highlights between cluster moves
# setEPS()
# postscript("OldSample2NEW.eps", width = 12, height = 8)
# par(cex.main = 2)
# plot( net , layout = l)
# 
# legend(x = "topright",
#        legend = c("Firms in main block","Within-block mobility"),
#        cex = 1.5, col = cols[1], bty = "n", lty = c("blank","solid"), pt.bg = c(cols[1],"transparent"), pch = c(21,NA_integer_)
# )
# 
# legend(x = "topleft",
#        legend = c("Firms in second block","Within-block mobility","Firms in third block","Within-block mobility"),
#        cex = 1.5, col = c(cols[2],cols[2],cols[3],cols[3]), bty = "n", lty = c("blank","solid","blank","solid"), pt.bg = c(cols[2],"transparent",cols[3],"transparent"), pch = c(21,NA_integer_)
# )
# 
# legend(x = "top",
#        legend = c("Between block mobility"),
#        cex = 1.5, col = c("black"), bty = "n", lty = c("solid")
# )
# dev.off()

#Graphical parameters
HighlightEdge <-(Edges$high1 + Edges$high2>=1)
E(net)$width <- .2 + .4*HighlightEdge
E(net)$color <- c(cols,"black")[1 + (Edges$cluster1from==1) * (Edges$cluster1to==1) * (HighlightEdge==0) 
                                + 2*(Edges$cluster2from==1) * (Edges$cluster2to==1) * (HighlightEdge==0) + 3*HighlightEdge]

# Save the network plot that highlights all high weight moves
postscript("OldSample2weightNEW.eps", width = 12, height = 8)
par(cex.main = 2)
plot( net , layout = l)

legend(x = "topright",
       legend = c("Firms in main block","Within-block mobility"),
       cex = 1.1, col = cols[1], bty = "n", lty = c("blank","solid"), pt.bg = c(cols[1],"transparent"), pch = c(21,NA_integer_)
)

legend(x = "topleft",
       legend = c("Firms in second block","Within-block mobility","Firms in third block","Within-block mobility"),
       cex = 1.1, col = c(cols[2],cols[2],cols[3],cols[3]), bty = "n", lty = c("blank","solid","blank","solid"), pt.bg = c(cols[2],"transparent",cols[3],"transparent"), pch = c(21,NA_integer_)
)


legend(x = "top",
       legend = c("High weight mobility"),
       cex = 1.1, col = c("black"), bty = "n", lty = c("solid")
)
dev.off()
